#include "player.hpp"
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include "board.hpp"

using namespace Voltmeter;


static void
setup_timer(uint16_t sample_rate)
{
    /* Fast PWM mode, TOP=ICR1, f=clk/1 */
    ICR1 = F_CPU / sample_rate - 1;
    OCR1B = ICR1 - 1;

    // TCCR1A = _BV(COM1B1) | _BV(COM1B0) | _BV(WGM11);
    TCCR1A = _BV(WGM11);
    TCCR1B = _BV(WGM13) | _BV(WGM12) | _BV(CS10);
    TCCR1C = 0x00;
    TIMSK1 = _BV(TOIE1);
}


static void
stop_timer()
{
    TIMSK1 = 0x00;
    TCCR1A = 0x00;
    TCCR1B = 0x00;
    TCCR1C = 0x00;
    ICR1 = 0x00;
    OCR1B = 0x00;
}


Player::Player()
    : fileOpen(false)
{
}


bool Player::MountFs()
{
    return (f_mount(&fs, "", 1) == FR_OK);
}


bool Player::OpenFile(const char *path)
{
    UINT bytes_read;
    uint16_t sample_rate;

    fileOpen = false;

    if (!f_open(&file, path, FA_READ) == FR_OK)
    {
        return false;
    }

    f_read(&file, (void *) PLAYBACK_BUFFER, WAV_HEADER_SIZE, &bytes_read);

    if (bytes_read != WAV_HEADER_SIZE)
    {
        f_close(&file);
        return false;
    }

    // NUM_CHANNELS = *((uint16_t *) &PLAYBACK_BUFFER[0x16]);
    sample_rate = *((uint16_t *) &PLAYBACK_BUFFER[0x18]);
    data_length = *((uint32_t *) &PLAYBACK_BUFFER[0x28]);

    memset(PLAYBACK_BUFFER, 0, sizeof(PLAYBACK_BUFFER));
    PLAYBACK_INDEX = PLAYBACK_BUFFER_LEN / 2;
    position = 0;
    offset = 0;

    setup_timer(sample_rate);

    fileOpen = true;
    return fileOpen;
}


bool Player::Play()
{
    if (!fileOpen)
    {
        return false;
    }

    uint16_t to_read;
    UINT bytes_read;
    
    if (data_length - position < PLAYBACK_BUFFER_LEN / 2)
    {
        to_read = data_length - position;
    }
    else
    {
        to_read = PLAYBACK_BUFFER_LEN / 2;
    }

    // Wait while player is playing at offset.
    while (PLAYBACK_INDEX >= offset && PLAYBACK_INDEX < offset + PLAYBACK_BUFFER_LEN / 2);

    f_read(&file, PLAYBACK_BUFFER + offset, to_read, &bytes_read);
    position += bytes_read;
    offset = (offset == 0) ? (PLAYBACK_BUFFER_LEN / 2) : 0;

    if (bytes_read != to_read || position == data_length)
    {
        while (PLAYBACK_INDEX >= offset && PLAYBACK_INDEX < offset + PLAYBACK_BUFFER_LEN / 2);
        offset = (offset == 0) ? (PLAYBACK_BUFFER_LEN / 2) : 0;
        while (PLAYBACK_INDEX >= offset && PLAYBACK_INDEX < offset + bytes_read);

        stop_timer();
        f_close(&file);
        fileOpen = false;

        return false;
    }

    return true;
}


bool Player::IsPlaying()
{
    return fileOpen;
}


uint16_t Player::GetProgress()
{
    if (!fileOpen || data_length == 0)
    {
        return 0xffff;
    }

    return (uint64_t) position * 10000 / data_length;
}


int16_t Player::GetSample()
{
    int16_t * sample = (int16_t *) (PLAYBACK_BUFFER + PLAYBACK_INDEX);
    int16_t result = *sample;

    PLAYBACK_INDEX += 2;

    if (PLAYBACK_INDEX == PLAYBACK_BUFFER_LEN)
    {
        PLAYBACK_INDEX = 0;
    }

    return result;
}